<?php
/******************************* PHP-P10 v2 *****************************
 * Copyright (C) 2011  Philipp Kreil (pk910)                            *
 *                                                                      *
 * This program is free software: you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation, either version 3 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      * 
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program. If not, see <http://www.gnu.org/licenses/>. *
 *                                                                      *
 ************************************************************************
 * 
 *  Bots/IPv6.class.php
 *
 * IPv6 bot...
 *
 */

class {$_NAME} extends Bot {
	private $uplink;
	private $ipv6;
	private $cache = array(
						0 => array(),
						1 => array()
					);
	
	public function load($uplink, $old = false) {
		$this->uplink = $uplink;
		if(!$old) {
			$nick = "IPv6";
			$ident = "ipv6";
			$ip = "fd00::c0ca:c01a:add5:11fe"; //coca cola adds life   maybe someone undestands it :D
			$host = $ip;
			$realname = "IPv6";
			$modes = "ioknISD";
			$this->ipv6 = $this->uplink->addUser($nick,$ident,$host,$ip,$modes,$realname);
			if(is_a($this->ipv6, "P10_User")) {
				$this->uplink->join($this->ipv6, "#IPv6", (P10_Channel::USERPRIV_OPED | P10_Channel::USERPRIV_VOICE));
				$this->uplink->join($this->ipv6, "#dev", P10_Channel::USERPRIV_VOICE);
			}
		} else {
			$this->ipv6 = $old;
		}
		
		ModCMD::bind($this, BIND_JOIN, "recive_join");
		ModCMD::bind($this, BIND_CHANMODE, "recive_mode");
		ModCMD::bind($this, BIND_KICK, "recive_kick");
		ModCMD::bind($this, BIND_QUIT, "recive_quit");
	}
	
	public function unload($rehash = false) {
		if($rehash) {
			return $this->ipv6;
		} else {
			$this->uplink->delUser($this->ipv6, "Bye.");
		}
	}
	
	public function loop() {
		for($i = 0; $i < 2; $i++) {
			foreach($this->cache[$i] as $id => $cache) {
				if($cache < time()) unset($this->cache[$i][$id]);
			}
		}
	}
	
	private function botOppedOnChannel($channel) {
		$privs = $channel->getUserPrivs($this->ipv6);
		return ($privs & P10_Channel::USERPRIV_OPED);
	}
	
	public function recive_join($user, $channel, $isBurst) {
		if(!$this->botOppedOnChannel($channel)) return false;
		if($user->getIP()->isIPv6()) {
			$this->uplink->mode($this->ipv6, $channel, "+ov ".$user->getNumeric()." ".$user->getNumeric());
		} else {
			$this->uplink->mode($this->ipv6, $channel, "+v ".$user->getNumeric());
		}
	}
	
	public function recive_mode($user, $channel, $modes) {
		if($user->getModes()->hasMode('o') || !$this->botOppedOnChannel($channel)) return false;
		if(array_key_exists($user->getNumeric(), $this->cache[0]) && $this->cache[0][$user->getNumeric()]['time'] > time()) {
			$this->uplink->kick($this->ipv6, $user, $channel, "you're not allowed to change modes in this channel!");
		} else {
			$this->uplink->notice($this->ipv6, $user, "you're not allowed to change modes in this channel!");
			$this->cache[0][$user->getNumeric()] = time() + 600;
		}
		$mode = explode(" ",$modes);
		$modestr="";
		$params="";
		$p=0;
		for($i=0;$i<strlen($mode[0]);$i++) {
			switch ($mode[0][$i]) {
				case "+":
					$mode[0][$i] = "-";
					break;
				case "-":
					$mode[0][$i] = "+";
					break;
			}
		}
		$modes = implode(" ",$mode);
		$this->uplink->mode($this->ipv6, $channel, $modes);
	}
	
	public function recive_kick($user, $target, $channel, $modes) {
		if($user->getModes()->hasMode('o') || !$this->botOppedOnChannel($channel)) return false;
		if(array_key_exists($user->getNumeric(), $this->cache[1]) && $this->cache[1][$user->getNumeric()]['time'] > time()) {
			$this->uplink->delUser($user, "You have been warned not to kick users from this Channel!");
		} else {
			$this->uplink->kick($this->ipv6, $user, $channel, "you're not allowed to kick users from this channel!");
			$this->cache[1][$user->getNumeric()] = time() + 600;
		}
	}
	
	public function recive_quit($user, $reason) {
		if($user === $this->ipv6) {
			$this->load($this->uplink);
		}
		for($i = 0; $i < 2; $i++) {
			if(array_key_exists($user->getNumeric(), $this->cache[$i])) {
				unset($this->cache[$i][$user->getNumeric()]);
			}
		}
	}
}

?>